/*
ROTARY SWITCH (ручка управления вид сверху)
 
 b1      +5
 |       |
 
 
 |   |   |
 b3  +5   b2
 
 Жестко заданы пины для аппаратного SPI:
 11 - IN  дисплея
 13 - CLK дисплея
 */

//Раскомментировать, если НЕ используется стандартный arduino модуль
/*
#define MAX7219_DIGIT0 0x01
#define MAX7219_DIGIT1 0x02
#define MAX7219_DIGIT2 0x03
#define MAX7219_DIGIT3 0x04
#define MAX7219_DIGIT4 0x05
#define MAX7219_DIGIT5 0x06
#define MAX7219_DIGIT6 0x07
#define MAX7219_DIGIT7 0x08
*/

//Раскомментировать, если Используется стандартный arduino модуль

#define MAX7219_DIGIT0 0x08
#define MAX7219_DIGIT1 0x07
#define MAX7219_DIGIT2 0x06
#define MAX7219_DIGIT3 0x05
#define MAX7219_DIGIT4 0x04
#define MAX7219_DIGIT5 0x03
#define MAX7219_DIGIT6 0x02
#define MAX7219_DIGIT7 0x01



const int  Pin_cs = 10;       //CS MAX7219
const int  Pin_b1 = 6;        //ROTARY 1
const int  Pin_b2 = 7;        //ROTARY 2
const int  Pin_b3 = 8;        //ROTARY 3
const int  Pin_voice = 3;     //ЗВУК
const int  Pin_relay = 2;     //РЕЛЕ

const int  Pin_mot1_1= 15;    //ДВИГАТЕЛЬ 2 ОБМОТКА 1
const int  Pin_mot1_2= 14;    //ДВИГАТЕЛЬ 2 ОБМОТКА 2
const int  Pin_mot1_3= 9;    //ДВИГАТЕЛЬ 2 ОБМОТКА 3
const int  Pin_mot1_4= 4;     //ДВИГАТЕЛЬ 2 ОБМОТКА 4

const int  Pin_mot2_1= 19;    //ДВИГАТЕЛЬ 1 ОБМОТКА 1
const int  Pin_mot2_2= 18;    //ДВИГАТЕЛЬ 1 ОБМОТКА 2
const int  Pin_mot2_3= 17;    //ДВИГАТЕЛЬ 1 ОБМОТКА 3
const int  Pin_mot2_4= 16;    //ДВИГАТЕЛЬ 1 ОБМОТКА 4

#define W1_PORT PORTD
#define W1_DDR DDRD
#define W1_PIN PIND
#define W1_BIT 5              //Датчик температуры пин №5

#include <SPI.h>
#include <EEPROM.h>
#include <Stepper.h>

#define STEPS_PER_MOTOR_REVOLUTION 32   
#define MOTOR_REV 32 * 64

#define MAX7219_MODE_DECODE 0x09
#define MAX7219_MODE_INTENSITY 0x0A
#define MAX7219_MODE_SCAN_LIMIT 0x0B
#define MAX7219_MODE_POWER 0x0C
#define MAX7219_MODE_NOOP 0x00


#define MAX7219_CHAR_BLANK 0xF
#define MAX7219_CHAR_NEGATIVE 0xA 

#define ON 1
#define OFF 0
#define NULL charTable[25]

Stepper motor1(STEPS_PER_MOTOR_REVOLUTION, Pin_mot2_1, Pin_mot2_3, Pin_mot2_2, Pin_mot2_4);

const int  set_up_max=14;

const static byte charTable[27] = {

  B01111110, //0    
  B00110000, //1
  B01101101, //2
  B01111001, //3
  B00110011, //4
  B01011011, //5
  B01011111, //6
  B01110000, //7
  B01111111, //8
  B01111011, //9
  B00001111, //t      10
  B00111101, //d      11
  B00010101, //n      12
  B00001110, //L      13
  B01001111, //E      14
  B01000111, //F      15
  B00000100, //i      16
  B01100011, //%up    17
  B00011101, //%down  18
  B01110111, //A      19
  B00011100, //u      20
  B00001101, //c      21
  B00011111, //b      22
  B00010111, //h      23
  B10000000, //.      24
  B00000000, //null   25
  B00000101  //r      26
};

const static char charSetup[set_up_max][3] = {
  {
    24,24,24          }
  ,   //  ...   - выход в основное меню 
  {
    10,5,10          }
  ,    //  tst 1 - температура начала отсчета времени работы на себя (в целых градусах)
  {
    10,11,12          }
  ,   //  tdn 2 - температура, с которой начинается уменьшение отбора (в целых градусах)
  {
    10,11,13          }
  ,   //  tdl 3 - диапазон температуры, за который снижается отбор (десятая градуса) 
  {
    10,14,22          }
  ,   //  teb 4 - температура, до которой отбираем тело (десятые градуса)
  {
    10,14,12          }
  ,   //  ten 5 - температура окончания процесса (десятые градуса) (сработка реле окончания процесса)
  {
    10,5,15          }
  ,    //  tsf 6 - время работы на себя (десятые часа)
  {
    10,16,11          }
  ,   //  tid 7 - время нахождения иглы внизу отбора для заполнения шланга перед началом отбора голов (десятые минут)
  {
    23,14,11          }
  ,   //  hed 8 - объем голов (мл) рассчитывается по данным времени отбора и скорости отбора, установленной при калибровке
  {
    17,18,11          }
  ,   //  %d  9 - уменьшение отбора в % на диапазон снижения (параметр 3)
  {
    19,20,10          }
  ,   //  aut 10 - время автостарта. если 0, то выключен, если больше 0, то это время бездействия, после которого прибор начнет работу с последнего использованного режима
  {
    5,21,13          }
  ,    //  scl 11 - шаг калибровки
  {
    13,23,11          }
  ,   //  lhd 12 - объем калибровки голов
  {
    13,22,11          }
  //  lbd 13 - объем калибровки тела
};

const static char charRej[4][3] = {
  {
    5,14,10          }
  ,    //  set 1 - настройки
  {
    21,19,13          }
  ,   //  cal 2 - калибровка
  {
    19,0,1          }
  ,   //  a01 3 - алгоритм 1. ректификация по Ткуба с линейным снижением отбора 
  {
    19,0,2          }    //  a02 4 - алгоритм 2. старт-стоп
};

const static char charVar[8][3] = {
  {
    23,14,11          }
  ,    //  hed
  {
    22,18,11          }
  ,   //  bod
  {
    15,5,25          }
  ,   //  FS  
  {
    5,10,22          }
  ,    //  Stb
  {
    22,20,10    }
  ,      //but
  {
    10,18,12    }
  ,      //ton
  {
    10,19,13    }
  ,       //tal
  {
    14,12,11    }

};





const static int par_min[set_up_max-1]=   {
  20,  20,   0, 200, 200, 0  ,0,  1,   1,   0, 1,   1,  10};
const static int par_max[set_up_max-1]=   {
  99,  99,  10, 990, 999, 99 ,99, 999, 100, 9, 999, 500, 500};
//const static char par_point[set_up_max-1]={0,   0,    1, 1,   1,   1,  1,  0,   0,   0, 0,   0,  0};
const static char par_point[set_up_max-1]={
  0,   0,    0, 0,   0,   0,  0,  0,   0,   0, 0,   0,  0};

const static char par_count[set_up_max-1]={
  2,   2,    2, 3,   3,  2,  2,  3,   3,   1, 3,   3,  3};

char rej=0,set_up=0,rej_max=3,lev=0,rej_step=0,tic=0;
int i=0,isbutton=0,Eaddress,temp_temp,is_start=0,curr_temp=0,rev_count=0,fact_rev_count=0,is_down=0,rev_total=0,rev_temp=0,rev_count_temp=0,
rev_curr=0,tmp02=0,head_count=0,body_count=0;
int buttonState,NewState,OldState,upState,downState;
int par[set_up_max-1];
long stab_time=0,kol_head=0,down_count=0;

void show_dis(int d1, int d2, int d3, int nn) {
  if (nn==1)
  {
    MAX7219_writeData(MAX7219_DIGIT0,charTable[d1]); 
    MAX7219_writeData(MAX7219_DIGIT1,charTable[d2]); 
    MAX7219_writeData(MAX7219_DIGIT2,charTable[d3]);              
  }
  else
  {
    MAX7219_writeData(MAX7219_DIGIT4,charTable[d1]); 
    MAX7219_writeData(MAX7219_DIGIT5,charTable[d2]); 
    MAX7219_writeData(MAX7219_DIGIT6,charTable[d3]);      
  } 
}


unsigned char w1_find() {
  unsigned char device;
  W1_DDR |= 1 << W1_BIT;
  delayMicroseconds(485);
  W1_DDR &= ~(1 << W1_BIT);
  delayMicroseconds(65);
  if ((W1_PIN & (1 << W1_BIT)) == 0x00)
    device = 1;
  else
    device = 0;
  delayMicroseconds(420);
  return device;
}

void w1_sendcmd(unsigned char cmd) {
  for (unsigned char i = 0; i < 8; i++) {
    if ((cmd & (1 << i)) == 1 << i) {
      W1_DDR |= 1 << W1_BIT;
      delayMicroseconds(2);
      W1_DDR &= ~(1 << W1_BIT);
      delayMicroseconds(65);
    } 
    else {
      W1_DDR |= 1 << W1_BIT;
      delayMicroseconds(65);
      W1_DDR &= ~(1 << W1_BIT);
      delayMicroseconds(5);
    }
  }
}

unsigned char w1_receive_byte() {
  unsigned char data = 0;
  for (unsigned char i = 0; i < 8; i++) {
    W1_DDR |= 1 << W1_BIT;
    delayMicroseconds(2);
    W1_DDR &= ~(1 << W1_BIT);
    delayMicroseconds(7);
    if ((W1_PIN & (1 << W1_BIT)) == 0x00)
      data &= ~(1 << i);
    else
      data |= 1 << i;
    delayMicroseconds(50);
  }
  return data;
}

int temp_18b20() {
  unsigned char data[2];
  int temp = 0;
  if (w1_find() == 1) {
    w1_sendcmd(0xcc);
    w1_sendcmd(0x44);
    delay(750);
    w1_find();
    w1_sendcmd(0xcc);
    w1_sendcmd(0xbe);
    data[0] = w1_receive_byte();
    data[1] = w1_receive_byte();
    temp = data[1];
    temp = temp << 8;
    temp |= data[0];
    temp = temp * 0.625;
    if (temp > 0 && temp < 990) {
      temp_temp = temp;
    } 
    else {
      temp = temp_temp;
    }
  }
  return temp;
}


int get_100(int data) {
  return data / 100;
}

int get_10(int data) {
  return (data - data / 100 * 100) / 10;
}

int get_1(int data) {
  return (data - data / 100 * 100) % 10;
}


void Eread()
{
  for (int j=1;j<set_up_max;j++)
  {
    int tmp1;
    tmp1 = EEPROM.read(j*2-1)*100+EEPROM.read(j*2);
    par[j-1] = tmp1;

    if (tmp1==0)
    {
      par[j-1] = par_min[j-1];
    }
    else
    {
      par[j-1] = tmp1;
    }

  }
}

void Ewrite()
{
  EEPROM.write(set_up*2-1, par[set_up-1]/100);
  EEPROM.write(set_up*2,  par[set_up-1]-par[set_up-1]/100*100);
}


void MAX7219_writeData(int address, int value) {
  // take the SS pin low to select the chip:
  digitalWrite(Pin_cs,LOW);
  //  send in the address and value via SPI:
  SPI.transfer(address);
  SPI.transfer(value);
  // take the SS pin high to de-select the chip:
  digitalWrite(Pin_cs,HIGH); 
}

void MAX7219_clearDisplay()
{
  char i = 7;
  // Loop until 0, but don't run for zero
  do {
    // Set each display in use to blank
    MAX7219_writeData(i, MAX7219_CHAR_BLANK);
  } 
  while (--i);
} 



ISR(TIMER2_COMPA_vect) 
{
  buttonState = digitalRead(Pin_b3)*2+digitalRead(Pin_b2)*4;

  if (digitalRead(Pin_b1)==1&isbutton==0)
  {

    if (is_start==0)
    {

      switch (lev)
      {
      case 0:
        {
          if (rej==0)
          {
            lev=1;
          }
          if (rej>0)
          {
            is_start=1;
            head_count=EEPROM.read(50)*100+EEPROM.read(51);
            body_count=EEPROM.read(52)*100+EEPROM.read(53);          
            stab_time=par[5]*6*600;
            kol_head=par[7]*36000/par[11];

          }

          break;    
        }
      case 1:
        {
          if (set_up==0)
          {
            lev=0;
          }
          else
          {
            lev=2;
          }
          break;    
        }
      case 2:
        {
          Ewrite();
          lev=1;
          break;    
        }

      } 
    }    
    else
    {
      switch (rej)
      {
      case 1:
        {
          switch (rej_step)
          {
          case 0:

            {
              EEPROM.write(50, rev_count/100);
              EEPROM.write(51,  rev_count-rev_count/100*100);   
              rej_step=1;
              break;
            }

          case 1:
            {

              EEPROM.write(52, rev_count/100);
              EEPROM.write(53,  rev_count-rev_count/100*100);   
              rej_step=0;
              rev_count=0;
              is_start=0;
              set_up=0;
              lev=0;
              rej=0;
              break;
            }

          }
          break;
        }
      case 2:
        {
          switch (rej_step)
          {
          case 3:

            {
              rej_step=4;
              fact_rev_count=0;                  
              rev_count = body_count;
              rev_curr=rev_curr+rev_count;
              rev_total = rev_count;
              rev_temp = (par[3]-par[1]*10+1)/par[2];
              rev_count_temp = rev_count/rev_temp;
              break;
            }

          case 5:
            {
              fact_rev_count=0;
              rev_count=body_count;
              rev_curr=rev_curr+rev_count;
              rej_step=6;
              break;
            }
          case 7:
            {
              rev_curr=0;
              rej_step=0;
              rev_count=0;
              is_start=0;
              set_up=0;
              lev=0;
              rej=0;
              break;
            }

          }
          break;
        }


      }
    }
    isbutton=1000;
    MAX7219_clearDisplay;
  }

  isbutton--;
  if (isbutton<1)
  {
    isbutton=0;
  }


  NewState=buttonState;
  if(NewState!=OldState)
  {
    switch(OldState)
    {
    case 4:
      {
        if(NewState == 6) upState++;
        if(NewState == 0) downState++; 
        break;
      }

    case 0:
      {
        if(NewState == 4) upState++;
        if(NewState == 2) downState++; 
        break;
      }
    case 2:
      {
        if(NewState == 0) upState++;
        if(NewState == 6) downState++; 
        break;
      }
    case 6:
      {
        if(NewState == 2) upState++;
        if(NewState == 4) downState++; 
        break;
      }
    }            
    OldState=NewState;
  }

  if (upState >= 4)
  {

    if (is_start>0&rej==1)
    {
      rev_count++;
    }

    if (is_start==0)
    {
      switch (lev)
      {
      case 0:
        {
          rej++;  
          if (rej>rej_max)
          {
            rej=0;
          }
          break;
        }
      case 1:
        {
          set_up++;  
          if (set_up>set_up_max-1)
          {
            set_up=0;
          }
          break;
        }
      case 2:
        {
          par[set_up-1]++;  
          if (par[set_up-1]>par_max[set_up-1])
          {
            par[set_up-1]=par_min[set_up-1];
          }
          break;
        }
      }
    }
    upState=0;
    MAX7219_clearDisplay;
  }

  if (downState >= 4)
  {
    if (is_start>0&rej==1)
    {
      if (rev_count>0)
      {
        rev_count--;
      }
    }

    if (is_start==0)
    {


      switch (lev)
      {
      case 0:
        {
          if (rej==0)
          {
            rej=rej_max+1;
          }
          rej--;  

          break;    
        }
      case 1:
        {
          if (set_up==0)
          {
            set_up=set_up_max;
          }
          set_up--;  

          break;    
        }
      case 2:
        {
          if (par[set_up-1]==par_min[set_up-1])
          {
            par[set_up-1]=par_max[set_up-1]+1;
          }
          par[set_up-1]--;  
          break;    
        }
      }
    }
    downState=0;
    MAX7219_clearDisplay;
  }


}


ISR(TIMER1_COMPA_vect) 
{
  if (tic>10)
  {
    tic=0;
    if (is_start>0&rej>1&rev_count==fact_rev_count)
    {
      curr_temp=temp_18b20();
    }
  }
  else
  {
    tic++;    
  }


  if (rej==2&is_start>0&rej_step==1)
  {
    if (stab_time>0)
    {
      stab_time--;
    }
    else
    {
      rej_step=2;
    }
  }
  if (rej==2&is_start>0&rej_step==2)
  {
    if (kol_head>0)
    {
      kol_head--;
    }
    else
    {
      fact_rev_count=0;    
      rev_count = -head_count;
      rev_curr=rev_curr+rev_count;      
      rej_step=3;
    }
  }
  if (rej==2&is_start>0&rej_step==2&is_down==2)
  {
    if (down_count>0)
    {
      down_count--;
    }
    else
    {
      is_down=3;
    }


  }

}


void set_display()
{

  if (is_start==0)
  {
    if (lev==0)
    {
      show_dis(charRej[rej][0],charRej[rej][1],charRej[rej][2],1);
      show_dis(25,25,25,2);
    }
    else
    {
      show_dis(charSetup[set_up][0],charSetup[set_up][1],charSetup[set_up][2],1);

      MAX7219_writeData(MAX7219_DIGIT3,NULL);    
      if (set_up==0)
      {
        show_dis(25,25,25,2);
      }
      else
      {
        if (par_count[set_up-1]>2)
        {
          MAX7219_writeData(MAX7219_DIGIT4,charTable[get_100(par[set_up-1])]);    
        }
        else
        {
          MAX7219_writeData(MAX7219_DIGIT4,NULL);    
        }

        if (par_count[set_up-1]>1)
        {
          if (par_point[set_up-1]==0)
          {
            MAX7219_writeData(MAX7219_DIGIT5,charTable[get_10(par[set_up-1])]);    
          }
          else
          {
            MAX7219_writeData(MAX7219_DIGIT5,B10000000|charTable[get_10(par[set_up-1])]);    
          }
        }
        else
        {
          MAX7219_writeData(MAX7219_DIGIT5,NULL);    
        }

        MAX7219_writeData(MAX7219_DIGIT6,charTable[get_1(par[set_up-1])]);    
      }
    }
  }
  else
  {
    if (rej==1)
    {
      if (rej_step==0)
      {
        show_dis(charVar[0][0],charVar[0][1],charVar[0][2],1);
      }
      if (rej_step==1)
      {
        show_dis(charVar[1][0],charVar[1][1],charVar[1][2],1);
      }

      show_dis(get_100(rev_count),get_10(rev_count),get_1(rev_count),2);
    }  

    if (rej==2)
    {
      if (rej_step==0)
      {
        show_dis(charVar[2][0],charVar[2][1],charVar[2][2],1);
        show_dis(get_100(curr_temp),get_10(curr_temp),get_1(curr_temp),2);
      } 
      if (rej_step==1)
      {
        show_dis(charVar[3][0],charVar[3][1],charVar[3][2],1);
        show_dis(get_100(stab_time/600),get_10(stab_time/600),get_1(stab_time/600),2);
      } 
      if (rej_step==2)
      {
        show_dis(charVar[0][0],charVar[0][1],charVar[0][2],1);
        show_dis(get_100(kol_head/600),get_10(kol_head/600),get_1(kol_head/600),2);
      } 
      if (rej_step==3|rej_step==5)
      {
        show_dis(charVar[4][0],charVar[4][1],charVar[4][2],1);
        show_dis(charVar[5][0],charVar[5][1],charVar[5][2],2);        
      } 
      if (rej_step==4)
      {
        show_dis(get_100(curr_temp),get_10(curr_temp),get_1(curr_temp),1);
        show_dis(get_100(rev_curr),get_10(rev_curr),get_1(rev_curr),2);
      } 
      if (rej_step==6)
      {
        show_dis(charVar[6][0],charVar[6][1],charVar[6][2],1);
        show_dis(get_100(curr_temp),get_10(curr_temp),get_1(curr_temp),2);
      } 
      if (rej_step==7)
      {
        show_dis(charVar[7][0],charVar[7][1],charVar[7][2],1);
        show_dis(25,25,25,2);
      } 

    }  

  }
  MAX7219_writeData(MAX7219_DIGIT3,NULL);    
  MAX7219_writeData(MAX7219_DIGIT7,NULL);    

}

void setup()
{
  pinMode(Pin_cs, OUTPUT);
  pinMode(Pin_b1, INPUT); 
  pinMode(Pin_b2, INPUT); 
  pinMode(Pin_b3, INPUT); 
  pinMode(Pin_voice, OUTPUT); 
  pinMode(Pin_relay, OUTPUT); 
  pinMode(Pin_mot1_1, OUTPUT); 
  pinMode(Pin_mot1_2, OUTPUT); 
  pinMode(Pin_mot1_3, OUTPUT); 
  pinMode(Pin_mot1_4, OUTPUT); 
  pinMode(Pin_mot2_1, OUTPUT); 
  pinMode(Pin_mot2_2, OUTPUT); 
  pinMode(Pin_mot2_3, OUTPUT); 
  pinMode(Pin_mot2_4, OUTPUT); 

  motor1.setSpeed(900);

  SPI.begin();

  MAX7219_writeData(MAX7219_MODE_SCAN_LIMIT, 7);
  MAX7219_writeData(MAX7219_MODE_INTENSITY, 5);
  MAX7219_writeData(MAX7219_MODE_POWER, ON);   
  MAX7219_clearDisplay();

  cli();
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  OCR1A = 1562;// = (1/10) / ((1/(16*10^6))*1024) 
  TCCR1B |= (1 << WGM12);
  TCCR1B |= (1 << CS10)|(1 << CS12);
  TIMSK1 |= (1 << OCIE1A);	

  TCCR2B = 0;
  TCNT2 = 0;
  OCR2A = 15;// = (1/1000) / ((1/(16*10^6))*1024) 
  TCCR2B |= (1 << CS10)|(1 << CS12);
  TIMSK2 |= (1 << OCIE2A);	
  sei();
  rej=0;
  Eread();

}


void loop()
{ 


  if (rev_count>fact_rev_count)
  {
    motor1.step(par[10]); 
    fact_rev_count++;
  }

  if (rev_count<fact_rev_count)
  {
    motor1.step(-par[10]); 
    fact_rev_count--;
  }


  ///////////////////////////////////////////////////////алгоритм 1 - ректификация по Ткуба
  if (rej==2&is_start>0&rej_step==2&is_down==0)
  {
    rev_count = body_count;
    rev_curr=rev_curr+rev_count;
    is_down=1;

  }

  if (rej==2&is_start>0&rej_step==2&is_down==1&rev_count==fact_rev_count)
  {
    down_count=par[6]*60;
    is_down=2;
  }

  if (rej==2&is_start>0&rej_step==2&is_down==3&rev_count==fact_rev_count&rev_count!=0)
  {
    fact_rev_count=0;
    rev_count = -(body_count-head_count);
    rev_curr=rev_curr+rev_count;
    is_down=4;

  }

  if (rej==2&is_start>0&rej_step==0&curr_temp>par[0]*10)
  {
    rej_step=1;
  }
  if (rej==2&is_start>0&rej_step==4&curr_temp>par[3])
  {
    rej_step=5;    
    fact_rev_count=0;
    rev_count=-rev_curr;
    rev_curr=rev_curr+rev_count;
  }
  tmp02=( body_count-(((curr_temp+par[2]-par[1]*10)*body_count*par[8])/(100*par[2])) );
  if (rej==2&is_start>0&rej_step==4&curr_temp<=par[3]&curr_temp>=par[1]*10&rev_count==fact_rev_count&
    rev_curr>tmp02)
  {
    fact_rev_count=0;
    rev_count= - (rev_curr-tmp02 );
    rev_curr=rev_curr+rev_count;
  }


  if (rej==2&is_start>0&rej_step==6&curr_temp>par[4])
  {
    rej_step=7;    
    fact_rev_count=0;
    rev_count=-body_count;
    rev_curr=rev_curr+rev_count;

  }

  if (rej==2)
  {
    if(is_start>0&(rej_step==3|rej_step==5))
    {
      digitalWrite(Pin_voice,HIGH);
    }
    else
    {
      digitalWrite(Pin_voice,LOW);
    }

  }
  if(rej==2&is_start>0&rej_step==7)
  {
    digitalWrite(Pin_relay,HIGH);
  }
  else
  {
    digitalWrite(Pin_relay,LOW);
  }



  //////////////////////////////////END/////////////////алгоритм 1 - ректификация по Ткуба









  set_display();
}





